#include "Cutscene.h"

#include <fstream>

#include "Video.h"

#include <cstdio>

bool Cutscene::Load(String path)
{
	Free();
	
	std::ifstream f( ("data/"+path).c_str());
	if(!f)
		return false;
	
	Scene *s;
	String dir_in;
	String dir_out;
	
	scenes.reserve(10);
	
	while(!f.fail())
	{
		s=new Scene();
		
		f >> s->path;
		f >> dir_in;
		f >> dir_out;
		f >> s->delay;
		
		s->anim_spd=0.75f;
		if(s->anim_spd>s->delay)
			s->anim_spd=s->delay;
			
		s->pct=0.0f;
		
		switch(dir_in[0])
		{
			case 'u': s->dir_in=UP; break;
			case 'd': s->dir_in=DOWN; break;
			case 'l': s->dir_in=LEFT; break;
			case 'r': s->dir_in=RIGHT; break;
		}
		
		switch(dir_out[0])
		{
			case 'u': s->dir_out=UP; break;
			case 'd': s->dir_out=DOWN; break;
			case 'l': s->dir_out=LEFT; break;
			case 'r': s->dir_out=RIGHT; break;
		}
		
		if(f.fail())
			delete s;
		else
			scenes.push_back(s);
	}
	
	if(scenes.size()<1)
		return false;	// Nie ma zadnej scenki a powinna jakas byc >:
		
	scene=0;
	finished=false;
	screenFilled=false;
	
	scenes[scene]->tex.Load(scenes[scene]->path);
	delay=scenes[scene]->delay;
	
	return true;
}
	
void Cutscene::Free()
{
	const unsigned size=scenes.size();
	for(unsigned i=0u; i<size; ++i)
	{
		scenes[i]->tex.Free();	// Bo czemu by nie~?
		delete scenes[i];
	}
	scenes.clear();
	
	scene=0u;
	delay=0.0f;
	finished=true;
	screenFilled=false;
}
	

void Cutscene::Update()
{
	if(IsFinished())
		return;
	finished=true;

	// extern float delta;

	delay-=delta;
	if(delay<=0.0f)
	{
		Next();
		screenFilled=true;
	}
		
	for(unsigned i=0u; i<scene; ++i)
	{
		Scene& s=*scenes[i];
		
		const float add=delta/s.anim_spd;
		
		s.pct+=add;
		
		if(s.pct>=1.0f)
			s.tex.Free();
		else
			finished=false;
	}
		
	if(scene<scenes.size())
	{
		Scene& s=*scenes[scene];
		
		const float add=delta/s.anim_spd;
		
		if(s.pct+add<0.5f)
			s.pct+=add;
		else
			s.pct=0.5f;
			
		finished=false;
	}
}
	
void Cutscene::Show()
{
	const float sw=Video::GetSize().w;
	const float sh=Video::GetSize().h;
	
	Video::DrawGUI(true);

	//Video::SetCamera(Vector());

	for(unsigned i=0u; i<scenes.size(); ++i)
	{
		Scene& s=*scenes[i];
		
		if(s.pct>=1.0f)
			continue;

		const float  w=s.tex.GetWidth();
		const float  h=s.tex.GetHeight();
		
		const Direction dir=(s.pct<=0.5f)?s.dir_in:s.dir_out;
		
		float x, y;
		
		switch(dir)
		{
			case UP:
				x=(sw-w)/2;
				y=sh*(1.0f-s.pct) - h*s.pct;
			break;
		
			case DOWN:
				x=(sw-w)/2;
				y=-h*(1.0f-s.pct) + sh*s.pct;
			break;
			
			case LEFT:
				y=(sh-h)/2;
				x=sw*(1.0f-s.pct) - w*s.pct;
			break;
			
			case RIGHT:
				y=(sh-h)/2;
				x=-w*(1.0f-s.pct) + sw*s.pct;
			break;
			
			default:
				x=(sw-w)/2;
				y=(sh-h)/2;
			break;
		}
		
		//printf("i %u; scene: %u; pct: %f; x, y: %f, %f\n", i, scene, s.pct, x, y);
		
		s.tex.Show(Rect(0, 0, w, h), Vector(x, y));
	}

	Video::DrawGUI(false);
}
	

void Cutscene::Skip()
	{
	if(scene==0 && scenes[0]->pct==0)
		{
		finished=true;
		scene=scenes.size();
		}
	else
		scene=scenes.size();
	}


void Cutscene::Next()
{
	//scenes[scene]->tex.Free();
	
	if(scene<scenes.size())
		++scene;
	
	if(scene>=scenes.size())
		return;
	
	scenes[scene]->tex.Load(scenes[scene]->path);
	delay=scenes[scene]->delay;
}

bool Cutscene::IsScreenFilled()
{
	return screenFilled;
}
