/*
 * Enemy.cpp
 *
 *  Created on: 24 sie 2013
 *     Project: LDJam27
 *      Author: cybek
 */

#include"Enemy.h"
#include<cstdlib>
#include<cmath>

#include"Player.h"
#include"Level.h"
#include"Particle.h"

#include"Input.h" // XXX
#include"Video.h" // XXX

Enemy::Enemy() : Object("Enemy"), ai(DEFEND), aitimer(0), firetimer(0)
{

}

Enemy::~Enemy()
{

}

bool Enemy::Init(String path, Vector nPos, Direction nDir, String nName, float nLife, WEAPON_TYPE nWeaponType, unsigned nWeaponAmmo)
{
	if(!spr.Load(path) )
		return false;

	if(!weapon.Init(this, nWeaponType, Vector(spr.GetMask().w/2.0f, spr.GetMask().h/4.0f) ) )
	//if(!weapon.Init(this, nWeaponType, Vector(26, 12) ) )
		return false;
	weapon.SetAmmo(nWeaponAmmo);

	pos=nPos;
	pos.z=Z_ENEMY;

	dir=nDir;
	spr.SetAnimation(GetDirectionName(dir) );

	name=nName;
	SetLife(nLife);

	if(name=="boss")
		ai=BOSS;
	else
		ai=DEFEND;

	aitimer=rand()/(RAND_MAX+1.0f);
	firetimer=0.0f;
	series=weapon.GetAmmoMax()/5;

	maskType=COL_RADIAL;
	//mask.x=spr.GetMask().x; mask.y=spr.GetMask().y;
	mask.x=0; mask.y=0;
	mask.w=spr.GetMask().w; mask.h=spr.GetMask().h;

	return true;
}

void Enemy::Quit()
{
	weapon.Quit();
	spr.Free();
}

void Enemy::Update()
{
	// NOTHING~!
	//if(name == "someone1")

	weapon.Update();
	if(ai==DEFEND)
	{
		UpdateAIDefend();
	}
	//if(name == "someone2")
	if(ai==BOSS)
	{
		UpdateAIBoss();
	}
}

void Enemy::UpdateAIDefend()
	{
	if(aitimer<=0.0f)
		{
		aitimer=1.0f;

		switch(dir)
			{
			case DIR_RIGHT:		dir=DIR_UP_RIGHT;	break;
			case DIR_UP_RIGHT:	dir=DIR_UP;			break;
			case DIR_UP:		dir=DIR_UP_LEFT;	break;
			case DIR_UP_LEFT:	dir=DIR_LEFT;		break;
			case DIR_LEFT:		dir=DIR_DOWN_LEFT;	break;
			case DIR_DOWN_LEFT:	dir=DIR_DOWN;		break;
			case DIR_DOWN:		dir=DIR_DOWN_RIGHT; break;
			case DIR_DOWN_RIGHT:dir=DIR_RIGHT;		break;
			}

		spr.SetAnimation(GetDirectionName(dir));
		}

	// Tymczasowe, do czasu zrobienia sprawdzania czy gracz widoczny
	//bool player_visible=!true, playerVisibleWithoutObjects=false;
	bool player_visible=true, playerVisibleWithoutObjects=true;
	float playerAngle=0.0f;

	/*
	 * BABYMETAL METHOD~!
	 */
	if(Level::player)
	{
		Vector v1=GetCenter(), v2=Level::player->GetCenter();
		float rad=atan2(v2.y-v1.y, v2.x-v1.x);
		playerAngle=rad/3.14f*180.0f;

		if(FindDirection(playerAngle) != dir)
		{
			player_visible=false;
			playerVisibleWithoutObjects=false;
		}
		else
		{
			//float lX=v1.x+cos(rad)*3, lY=v1.y+sin(rad)*3;
			float lX=v1.x, lY=v1.y;
			std::list<Object*> beamObjs;
			while(true) // for(;;)    :|
			{
				if(Level::CheckCollision(this, Rect(lX, lY, 1, 1), &beamObjs) )
				{
					if(Level::CheckCollisionOnlyLevel(this, Rect(lX, lY, 1, 1) ) )
					{
						beamObjs.clear();
						playerVisibleWithoutObjects=false;
						player_visible=false;
						break;
					}

					for(auto itr : beamObjs)
					{
						if(itr->GetObjectType() == "Player")
						{
							lX=v2.x;
							lY=v2.y;
							break;
						}
						else if(itr->GetObjectType() != "Bullet")
							player_visible=false;
					}
					beamObjs.clear();
				}
				lX+=cos(rad);
				lY+=sin(rad);

				if(Keys::IsPressed(SDLK_e) ) // bajeranckie :O
				{
					Level::Show();
					Video::DrawGUI(true);
					Video::DrawRect(Rect(lX-1-Video::GetCamera().x, lY-1-Video::GetCamera().y, 3, 3), Color(1.0f, 0.0f, 0.0f, 1.0f) );
					Video::DrawGUI(false);
					Video::Flip();
				}

				if(v1.x < v2.x && lX >= v2.x)
				{
					//printf("  BreakA\n");
					break;
				}
				if(v1.x > v2.x && lX <= v2.x)
				{
					//printf("  BreakB\n");
					break;
				}
				if(v1.y < v2.y && lY >= v2.y)
				{
					//printf("  BreakC\n");
					break;
				}
				if(v1.y > v2.y && lY <= v2.y)
				{
					//printf("  BreakD\n");
					break;
				}
			}
		}

		//float lDX=lX-v1.x, lDY=lY-v1.y;
		//float length=sqrt(lDX*lDX+lDY*lDY);
		//v2.x=v1.x+length*cos(rad);
		//v2.y=v1.y+length*sin(rad);
		//Video::DrawLine(v1, v2, 1, Color(1.0f, 0.0f, 0.0f, 0.5f) );//fabs(sin(GetTicks()/800.0f) )/2.0f+0.5f) );
	}
	else
	{
		player_visible=false;
		playerVisibleWithoutObjects=false;
	}

	// TODO WTF HOW~!?
	//printf("Visible: %d, Objs: %d, Angle: %.1f\n", player_visible, playerVisibleWithoutObjects, playerAngle);

	/*
	 * END OF BABYMETAL~!
	 */

	if(player_visible)
		{
		firetimer-=delta;

		if(series>0)
			{
			weapon.Wave(0.25f);

			if(firetimer<=0.0f)
				{
				firetimer=weapon.GetFireDelay()/1000.0f;
				if(firetimer<0.10f)
					firetimer=0.10f;

				weapon.Fire();
				weapon.AddAmmo(1);

				--series;
				}

			if(series==0)
				firetimer=2.0f;
			}
		else
			{
			if(firetimer<=0.0f)
				series=weapon.GetAmmoMax()/5;
			}
		}
	else
		aitimer-=delta;
	}

void Enemy::UpdateAIBoss()
	{
	Player* p=Level::player;

	if(!p)
		return;

	const Vector ppos=p->GetPosition();
	const Rect prect=p->GetRect();

	const float rad=atan2( (ppos.y+prect.y+prect.h/2)-(pos.y+mask.y+mask.h/2), (ppos.x+prect.x+prect.w/2)-(pos.x+mask.x+mask.w/2) );
	const float angle=rad/3.14f*180.0f;

	if(angle < 23 && angle > -23)
		dir=DIR_RIGHT;
	else if(angle < 68 && angle >= 23)
		dir=DIR_DOWN_RIGHT;
	else if(angle < 113 && angle >= 68)
		dir=DIR_DOWN;
	else if(angle < 158 && angle >= 113)
		dir=DIR_DOWN_LEFT;
	else if(angle < -158 || angle >= 158)
		dir=DIR_LEFT;
	else if(angle < -113 && angle >= -158)
		dir=DIR_UP_LEFT;
	else if(angle < -68 && angle >= -113)
		dir=DIR_UP;
	else if(angle < -23 && angle >= -68)
		dir=DIR_UP_RIGHT;

	spr.SetAnimation(GetDirectionName(dir));

	aitimer-=delta;
	if(aitimer<=4.0f)
		{
		if(aitimer<=0.0f)
			aitimer=10.0f;

		bool player_visible=true;

		if(player_visible)
			{
			weapon.Wave(0.4f);

			firetimer-=delta;
			if(firetimer<=0.0f)
				{
				firetimer=weapon.GetFireDelay()/1000.0f;
				if(firetimer<0.05f)
					firetimer=0.05f;

				weapon.Fire();
				weapon.AddAmmo(1);	// Czity, amunicja nigdy sie nie skonczy~a
				}
			}
		}
	}

void Enemy::Show()
{
	weapon.Show();
	spr.Show(pos);
}

void Enemy::OnDead()
{
	// TODO set dead animation?
	// dead corpses will be obstacle, so maybe create it?
	// NOPE
	// set anim to dead and life to -1.0f

	unsigned bloodAmount=mask.w*mask.h/10;
	//printf("Blood: %d\n", bloodAmount);
	for(unsigned a=0; a < bloodAmount; ++a)
	{
		Particle *p=new Particle;
		if(!p->Init(pos+Vector(mask.x+mask.w/2.0f, mask.y+mask.h/2.0f), (rand()%30)/20.0f+1.0f, rand()%360, (mask.w/2.0f+mask.h/2.0f), Color(1.0f, 0.0f, 0.0f, 1.0f), Color(0.6f, 0.0f, 0.0f, 0.0f) ) )
			printf("Can't create particle!\n");
		Level::AddObject(p);
	}
}
