/*
 * HUD.cpp
 *
 *  Created on: 26 sie 2013
 *     Project: LDJam27
 *      Author: cybek
 */

#include"Video.h"
#include"Level.h"
#include"Player.h"

namespace HUD
{
	Texture hud, hudMarker, counterBig, counterSmall, counterBigBG;
	Vector hudPos, counterPos, counterBGPos;
	unsigned counterBigWidth, counterSmallWidth;
	Sprite alarm;

	Vector hudWeaponOffset[WEAPON_COUNT];

	bool Init()
	{
		if(!hud.Load("gui.png") )
			return false;
		if(!hudMarker.Load("gui_marker.png") )
			return false;
		if(!counterBig.Load("counter-big.png") )
			return false;
		if(!counterSmall.Load("counter-small.png") )
			return false;
		if(!counterBigBG.Load("gui_big.png") )
			return false;

		if(!alarm.Load("alarm.spr") )
			return false;

		Size wnd=Video::GetSize();
		Size hudSize=hud.GetSize();
		hudPos=Vector(wnd.w/2.0f-hudSize.w/2.0f, wnd.h-hudSize.h);
		counterPos=Vector(wnd.w/2.0f-2*12, 8);

		counterBGPos=Vector(wnd.w/2.0f-counterBigBG.GetWidth()/2.0f, 0);

		counterBigWidth=24;
		counterSmallWidth=12;

		hudWeaponOffset[WEAPON_PISTOL]=Vector(66, 4);
		hudWeaponOffset[WEAPON_PPSZ]=Vector(138, 4);
		hudWeaponOffset[WEAPON_RCLR]=Vector(210, 4);
		hudWeaponOffset[WEAPON_CROSSBOW]=Vector(282, 4);
		hudWeaponOffset[WEAPON_EGGPLANT]=Vector(354, 4);

		return true;
	}

	void Quit()
	{
		alarm.Free();
		counterBigBG.Free();
		counterSmall.Free();
		counterBig.Free();
		hudMarker.Free();
		hud.Free();
	}

	void Show()
	{
		Video::DrawGUI(true);

		/*Player *player=Level::player;

		if(player)
		{
			//if(player->GetCurrentWeapon() == a)
			Vector dst=hudPos+hudWeaponOffset[player->GetCurrentWeapon() ];
			hudMarker.Show(Rect(0.0f, 0.0f, hudMarker.GetWidth(), hudMarker.GetHeight() ), Vector(dst.x-3*12-hudMarker.GetWidth(), dst.y) );
		}*/

		Size hudSize=hud.GetSize();
		hud.Show(Rect(0, 0, hudSize.w, hudSize.h), hudPos);

		/*
		 * DrawAmmo
		 */
		Player *player=Level::player;
		if(player)
		{
			// Pistol
			for(unsigned a=0; a < WEAPON_COUNT; ++a)
			{
				WEAPON_TYPE type=(WEAPON_TYPE)a;
				if(player->HasWeapon(type) )
				{
					Vector dst=hudPos+hudWeaponOffset[type];
					unsigned amount=player->GetAmmo(type);

					if(player->GetCurrentWeapon() == a)
						hudMarker.Show(Rect(0.0f, 0.0f, hudMarker.GetWidth(), hudMarker.GetHeight() ), Vector(dst.x-3*counterSmallWidth-hudMarker.GetWidth(), dst.y) );

					Rect src;
					src.y=0;
					src.w=counterSmallWidth;
					src.h=24;

					do
					{
						src.x=(amount%10+1)*counterSmallWidth;
						counterSmall.Show(src, dst);
						dst.x-=counterSmallWidth;
						amount/=10;
					} while(amount > 0);
				}
			}
		}

		/*
		 * Draw timer
		 */

		//Size counterBigBGSize=counterBigBG.GetSize();
		//counterBigBG.Show(Rect(0, 0, counterBigBGSize.w, counterBigBGSize.h), Vector(Video::GetSize().w/2.0f-counterBigBGSize.w/2.0f, 0) );
		counterBigBG.Show(Rect(0, 0, counterBigBG.GetWidth(), counterBigBG.GetHeight() ), counterBGPos);

		if(Level::alertTimerStarted)
		{
			unsigned elapsed=Level::alertTimer;
			alarm.Update();

			if(elapsed <= 10000) // FIRST ALAM
			{
				if(alarm.GetAnimation() != "yellow")
					alarm.SetAnimation("yellow");

				unsigned time=1000-elapsed/10;
				Vector dst=counterPos;
				dst.x+=2*counterBigWidth;

				Rect src;
				src.y=0;
				src.w=counterBigWidth;
				src.h=counterBig.GetHeight();

				for(unsigned a=0; a < 4; ++a)
				{
					if(time == 0 && a > 3)
						src.x=0;
					else
						src.x=(time%10+1)*counterBigWidth;
					counterBig.Show(src, dst);

					if(a == 1)
						counterBig.Show(Rect(11*counterBigWidth, 0, counterBigWidth, counterBig.GetHeight() ), dst);

					dst.x-=counterBigWidth;
					time/=10;
				}

				// Draw alarm
				if(Level::targetsCount > 0)
				{
					//Size wnd=Video::GetSize();
					alarm.Show(Vector(counterBGPos.x+counterBigBG.GetWidth(), 0.0f) );
					alarm.Show(Vector(counterBGPos.x-alarm.GetMask().w, 0.0f) );
				}
			}
			else // SECOND ALAM
			{
				if(alarm.GetAnimation() != "red")
					alarm.SetAnimation("red");

				unsigned time;
				if(elapsed <= 20000)
					time=2000-elapsed/10;
				else
					time=0;
				Vector dst=counterPos;
				dst.x+=2*counterBigWidth;

				Rect src;
				src.y=0;
				src.w=counterBigWidth;
				src.h=counterBig.GetHeight();

				for(unsigned a=0; a < 4; ++a)
				{
					if(time == 0 && a > 3)
						src.x=0;
					else
						src.x=(time%10+1)*counterBigWidth;
					counterBig.Show(src, dst);

					if(a == 1)
						counterBig.Show(Rect(11*counterBigWidth, 0, counterBigWidth, counterBig.GetHeight() ), dst);

					dst.x-=counterBigWidth;
					time/=10;
				}

				// Draw alarm
				//Size wnd=Video::GetSize();
				//alarm.Show(Vector(wnd.w/2.0f+2*counterBigWidth, 0.0f) );
				//alarm.Show(Vector(wnd.w/2.0f-2*counterBigWidth-alarm.GetMask().w, 0.0f) );
				alarm.Show(Vector(counterBGPos.x+counterBigBG.GetWidth(), 0.0f) );
				alarm.Show(Vector(counterBGPos.x-alarm.GetMask().w, 0.0f) );
			}
		}
		else // NO ALAM
		{
			unsigned time=1000;
			Vector dst=counterPos;
			dst.x+=2*counterBigWidth;

			Rect src;
			src.y=0;
			src.w=counterBigWidth;
			src.h=counterBig.GetHeight();

			for(unsigned a=0; a < 4; ++a)
			{
				//src.x=0;
				src.x=(time%10+1)*counterBigWidth;
				counterBig.Show(src, dst);

				if(a == 1)
					counterBig.Show(Rect(11*counterBigWidth, 0, counterBigWidth, counterBig.GetHeight() ), dst);

				dst.x-=counterBigWidth;
				time/=10;
			}
		}

		Video::DrawGUI(false);
	}
}
