/*
 * Level.cpp
 *
 *  Created on: 24 sie 2013
 *     Project: LDJam27
 *      Author: cybek
 */

#include"Level.h"
#include"Video.h"
#include"Texture.h"
#include"Input.h"
#include"Cutscene.h"
#include"HUD.h"
#include"main.h"
#include<fstream>
#include<vector>
#include<cmath>
#include<list>

/*
 *
 */
#include"Player.h"
#include"Enemy.h"
#include"Obstacle.h"
#include"Ammo.h"
#include"Target.h"
#include"TargetArea.h"

class EnemyAlert
{
	public:
		String path;
		Vector pos;
		String dir;
		String name;
		float life;
		String weapon;
		unsigned ammo;
};

namespace Level
{
	Texture tiles, grass;//, cursor;
	unsigned *level, levelW, levelH;
	std::vector<Object*> objs;
	std::list<EnemyAlert> enemyAlert;
	std::string cutIntro, cutOutro;
	Cutscene cutScene;
	bool /*cursorInSight=false,*/ drawLifes=true;
	Player *player=NULL; // NULL if player is dead~!~!~!
	unsigned targetsCount=0, alertTimer=0;
	bool alertTimerStarted=false;
	String curLevel;
	Rect nextLevelPos;
	String nextLevel;

	std::list<String> queueCutscenes;
	bool needNextLevel=false;

	enum GAME_STATE{STATE_GAME_INTRO, STATE_GAME_GAME, STATE_GAME_OUTRO};
	GAME_STATE gameState;

	//unsigned idEmpty[]={ 0 };
	unsigned idEmpty[]={ 1, 2, 9, 10 };
	unsigned idDestroy[]={  };
	float enemyAlertDist=64.0f;

	bool Init()
	{
		//if(!tiles.Load("tiles.png") )
		if(!tiles.Load("tileset_01-wooden.png") )
			return false;

		if(!grass.Load("grass.png") )
			return false;

		//if(!cursor.Load("cursor.png") )
		//	return false;

		if(!HUD::Init() )
			return false;

		return true;
	}

	void Quit()
	{
		Free();
		HUD::Quit();
		//cursor.Free();
		grass.Free();
		tiles.Free();
	}

	bool Load(String path)
	{
		Free();

		std::ifstream f( ("data/"+path).c_str() );
		if(!f)
			return false;
		curLevel=path;

		f >> levelW >> levelH;
		level=new unsigned[levelW*levelH];

		for(unsigned a=0; a < levelW*levelH; ++a)
			f >> level[a];
		if(f.fail() )
			return false;

		std::string cmd;
		while(true)
		{
			f >> cmd;
			if(f.fail() )
				break;

			if(cmd == "player")
			{
				Vector pos;
				float speed;
				bool weaponsEnabled[WEAPON_COUNT];
				f >> pos.x >> pos.y;
				f >> speed;
				for(unsigned a=0; a < WEAPON_COUNT; ++a)
					f >> weaponsEnabled[a];

				Player *p=new Player;
				if(!p->Init(pos, speed, weaponsEnabled) )
					return false;
				//objs.push_back(p);
				AddObject(p);

				p->Update(); // XXX, center camera on player~!
				player=p;
			}
			else if(cmd == "enemy")
			{
				String path, dir, name, weapon;
				Vector pos;
				float life;
				unsigned weaponAmmo;
				f >> path >> pos.x >> pos.y >> dir >> name >> life >> weapon >> weaponAmmo;

				Enemy *e=new Enemy;
				if(!e->Init(path, pos, GetDirection(dir), name, life, GetWeaponType(weapon), weaponAmmo) )
					return false;
				//objs.push_back(e);
				AddObject(e);

				e->Update();
			}
			else if(cmd == "enemyalert")
			{
				enemyAlert.push_back(EnemyAlert() );
				EnemyAlert &a=enemyAlert.back();
				f >> a.path >> a.pos.x >> a.pos.y >> a.dir >> a.name >> a.life >> a.weapon >> a.ammo;

				/*Enemy *e=new Enemy;
				if(!e->Init(path, pos, GetDirection(dir), name, life, GetWeaponType(weapon), weaponAmmo) )
					return false;
				//objs.push_back(e);
				AddObject(e);

				e->Update();*/
			}
			else if(cmd == "obstacle")
			{
				String path;
				Vector pos;
				float life;
				f >> path >> pos.x >> pos.y >> life;

				Obstacle *o=new Obstacle;
				if(!o->Init(path, pos, life) )
					return false;
				AddObject(o);
			}
			else if(cmd == "ammo")
			{
				Vector pos;
				String type;
				unsigned amount;
				f >> pos.x >> pos.y >> type >> amount;

				Ammo *a=new Ammo;
				if(!a->Init(pos, GetWeaponType(type), amount) )
					return false;
				AddObject(a);
			}
			else if(cmd == "target")
			{
				Vector pos;
				String path;
				float life;
				f >> path >> pos.x >> pos.y >> life;

				Target *t=new Target;
				if(!t->Init(path, pos, life) )
					return false;
				AddObject(t);

				targetsCount++;
			}
			else if(cmd == "targetarea")
			{
				Vector pos;
				Size size;
				f >> pos.x >> pos.y >> size.w >> size.h;

				TargetArea *a=new TargetArea;
				if(!a->Init(pos, size) )
					return false;
				AddObject(a);
			}
			else if(cmd == "nextlevel")
			{
				f >> nextLevel;
				f >> nextLevelPos.x >> nextLevelPos.y >> nextLevelPos.w >> nextLevelPos.h;

				if(nextLevel == "end")
					nextLevel.clear();
			}
			// Cutscene
			else if(cmd == "intro")
			{
				f >> cutIntro;
			}
			else if(cmd == "outro")
			{
				f >> cutOutro;
			}
			else
			{
				printf("Unknown command: %s\n", cmd.c_str() );
			}

			if(f.fail() )
				return false;
		}

		if(!cutIntro.empty() )
		{
			//if(!cutScene.Load(cutIntro) )
			//	return false;
			queueCutscenes.push_back(cutIntro);
		}
		else
			gameState=STATE_GAME_GAME;

		needNextLevel=false;

		delta=0.0f;
		deltaTimer=GetTicks();
		return true;
	}

	void Free()
	{
		if(level)
		{
			delete [] level;
			level=NULL;
		}

		for(auto itr : objs)
		{
			itr->Quit();
			delete itr;
		}
		objs.clear();

		cutIntro.clear();
		cutOutro.clear();

		player=NULL;
		targetsCount=0;
		alertTimerStarted=false;
		alertTimer=0;
		nextLevel.clear();
		gameState=STATE_GAME_INTRO;

		needNextLevel=false;

		enemyAlert.clear();
	}

	bool Reload()
	{
		bool result=Load(curLevel);
		//cutScene.Skip();
		queueCutscenes.clear();
		return result;
	}

	bool NextLevel()
	{
		if(!nextLevel.empty() )
			return Load(nextLevel);
		else
		{
			mainState=STATE_MAIN_MENU;
			Quit();
			return true;
		}
	}

	void AddObject(Object *o)
	{
		objs.push_back(o);
	}

	bool CanMove(unsigned id)
	{
		for(auto itr : idEmpty)
			if(id == itr)
				return true;

		return false;
	}

	/*bool CanDestroy(unsigned id)
	{
		for(auto itr : idDestroy)
			if(id == itr)
				return true;
		return false;
	}*/

	bool CheckCollision(Rect rA, Rect rB)
	{
		if(rA.x >= rB.x+rB.w || rA.y >= rB.y+rB.h || rA.x+rA.w <= rB.x || rA.y+rA.h <= rB.y)
			return false;
		else
			return true;
	}

	bool CheckCollisionOnlyLevel(Object *o, Rect rA)
	{
		if(rA.x < 0.0f || rA.y < 0.0f || rA.x+rA.w >= levelW*32 || rA.y+rA.h >= levelH*32)
		{
			//o->SetLife(0.0f);
			return true;
		}

		if(!CanMove( level[int(rA.y)/32*levelW+int(rA.x)/32 ] ) )
			return true;
		if(!CanMove( level[int(rA.y)/32*levelW+int(rA.x+rA.w-1)/32 ] ) )
			return true;
		if(!CanMove( level[int(rA.y+rA.h-1)/32*levelW+int(rA.x)/32 ] ) )
			return true;
		if(!CanMove( level[int(rA.y+rA.h-1)/32*levelW+int(rA.x+rA.w-1)/32 ] ) )
			return true;

		return false;
	}

	bool CheckCollision(Object *o, Rect rA, std::list<Object*> *objList)
	{
		if(rA.x < 0.0f || rA.y < 0.0f || rA.x+rA.w >= levelW*32 || rA.y+rA.h >= levelH*32)
		{
			//o->SetLife(0.0f);
			return true;
		}

		if(!CanMove( level[int(rA.y)/32*levelW+int(rA.x)/32 ] ) )
			return true;
		if(!CanMove( level[int(rA.y)/32*levelW+int(rA.x+rA.w-1)/32 ] ) )
			return true;
		if(!CanMove( level[int(rA.y+rA.h-1)/32*levelW+int(rA.x)/32 ] ) )
			return true;
		if(!CanMove( level[int(rA.y+rA.h-1)/32*levelW+int(rA.x+rA.w-1)/32 ] ) )
			return true;

		for(auto itr : objs)
		{
			if(itr == o || itr == o->GetParent() )//|| itr->GetObjectType() == "Particle")// || (itr->GetParent() != NULL && itr->GetParent() == o->GetParent() ) )
				continue;

			if(itr->GetMaskType() == COL_NONE)
				continue;

			//printf("Itr: %p, Itr->Par: %p, o: %p, O->Par: %p\n", itr, itr->GetParent(), o, o->GetParent() );

#ifdef _DEBUG_
			//if( (itr->GetRect().w == 0.0f || itr->GetRect().h == 0.0f) )//&& itr->GetObjectType() != "Particle")
			//	printf("Object without mask: %p (%s)\n", itr, itr->GetObjectType().c_str() );
#else
			if(itr->GetRect().w == 0.0f || itr->GetRect().h == 0.0f)
				continue;
#endif

			Rect rB=itr->GetRect()+itr->GetPosition();

			/*if(dynamic_cast<Obstacle*>(itr) )
			{
				printf("Player: %.1fx%.1f, Table: %.1fx%.1f, %.1fx%.1f\n", rA.x, rA.y, rB.x, rB.y, rB.w, rB.h);
			}*/

			if(CheckCollision(rA, rB) )
			{
				if(o->GetMaskType() == COL_RECT || itr->GetMaskType() == COL_RECT)
				{
					if(objList)
						objList->push_back(itr);
					return true;
				}

				// Circle collision
				float dx, dy;
				dx=(rA.x+rA.w/2)-(rB.x+rB.w/2);
				dy=(rA.y+rA.h/2)-(rB.y+rB.h/2);
				float dist=dx*dx+dy*dy;
				float mR=( (rA.w/2+rB.w/2)+(rA.h/2+rB.h/2) )/2;
				//printf("Dist: %.1f, %.1f\n", sqrt(dist), sqrt(mR*mR) );
				if(dist < mR*mR)
				{
					//printf("Col obj type: %s\n", itr->GetObjectType().c_str() );
					if(objList)
						objList->push_back(itr);
					return true;
				}
			}
		}

		return false;
	}

	bool MoveObject(Object *o, Vector &vel, bool stopAftCol, std::list<Object*> *objList)
	{
		bool fullMove=true;

		//Vector vel=o->GetVelocity();
		Rect rect=o->GetRect()+o->GetPosition();

		while(vel.x != 0.0f || vel.y != 0.0f)
		{
			if(vel.x != 0.0f)
			{
				float step=vel.x;
				if(fabs(vel.x) >= 1.0f)
				{
					if(vel.x > 0.0f)
						step=1.0f;
					else
						step=-1.0f;
				}

				if(!CheckCollision(o, rect+Vector(step, 0.0f), objList) )
				{
					rect.x+=step;
					vel.x-=step;
				}
				else
				{
					fullMove=false;
					if(stopAftCol)
						break;
					else
						vel.x-=step;
				}
			}

			if(vel.y != 0.0f)
			{
				float step=vel.y;
				if(fabs(vel.y) >= 1.0f)
				{
					if(vel.y > 0.0f)
						step=1.0f;
					else
						step=-1.0f;
				}

				if(!CheckCollision(o, rect+Vector(0.0f, step), objList) )
				{
					rect.y+=step;
					vel.y-=step;
				}
				else
				{
					fullMove=false;
					if(stopAftCol)
						break;
					else
						vel.y-=step;
				}
			}
		}

		Rect tmp=o->GetRect();
		o->SetPosition(Vector(rect.x-tmp.x, rect.y-tmp.y, o->GetPosition().z) );

		return fullMove;
	}

	/*void GetObjects(Object *o, Vector vel, std::list<Object*> &objList)
	{
		Rect rect=o->GetRect()+o->GetPosition();

		while(vel.x != 0.0f || vel.y != 0.0f)
		{
			if(vel.x != 0.0f)
			{
				float step=vel.x;
				if(fabs(vel.x) >= 1.0f)
				{
					if(vel.x > 0.0f)
						step=1.0f;
					else
						step=-1.0f;
				}
				vel.x-=step;

				if(!CheckCollision(o, rect+Vector(step, 0.0f) ) )
					rect.x+=step;
				else
				{
					for(auto itr : objs)
					{
						if(itr == o)
							continue;

						Rect rB=itr->GetRect()+itr->GetPosition();
						if(CheckCollision(rect, rB) )
						{
							// Circle collision
							float dx, dy;
							dx=(rect.x+rect.w/2)-(rB.x+rB.w/2);
							dy=(rect.y+rect.h/2)-(rB.y+rB.h/2);
							float dist=dx*dx+dy*dy;
							float mR=( (rect.w/2+rB.w/2)+(rect.h/2+rB.h/2) )/2;
							if(dist <= mR*mR)
								objList.push_back(itr);
						}
					}

					rect.x+=step;
				}
			}

			if(vel.y != 0.0f)
			{
				float step=vel.y;
				if(fabs(vel.y) >= 1.0f)
				{
					if(vel.y > 0.0f)
						step=1.0f;
					else
						step=-1.0f;
				}
				vel.y-=step;

				if(!CheckCollision(o, rect+Vector(0.0f, step) ) )
					rect.y+=step;
				else
				{
					rect.x+=step;

					for(auto itr : objs)
					{
						if(itr == o)
							continue;

						Rect rB=itr->GetRect()+itr->GetPosition();
						if(CheckCollision(rect, rB) )
						{
							// Circle collision
							float dx, dy;
							dx=(rect.x+rect.w/2)-(rB.x+rB.w/2);
							dy=(rect.y+rect.h/2)-(rB.y+rB.h/2);
							float dist=dx*dx+dy*dy;
							float mR=( (rect.w/2+rB.w/2)+(rect.h/2+rB.h/2) )/2;
							if(dist <= mR*mR)
								objList.push_back(itr);
						}
					}
				}
			}
		}
	}*/

	void GetObjects(Object *o, Vector pos, float radius, std::list<Object*> &objList)
	{
		for(auto itr : objs)
		{
			if(itr == o)
				continue;
			if(itr->GetMaskType() == COL_NONE)
				continue;

			Rect rB=itr->GetRect();
			Vector bB=itr->GetPosition()+Vector(rB.x+rB.w/2.0f, rB.y+rB.h/2.0f);
			float dx=bB.x-pos.x;
			float dy=bB.y-pos.y;
			float dist=sqrt(dx*dx+dy*dy);

			if(dist <= radius)
				objList.push_back(itr);
		}
	}

	void Update()
	{
		/*if(gameState == STATE_GAME_INTRO)
		{
			if(!cutScene.IsFinished() )
			{
				cutScene.Update();

				printf("Filled: %d\n", cutScene.IsScreenFilled() );

				if(Keys::IsPressed(SDLK_ESCAPE) )
					cutScene.Skip();
				return;
			}
			else
				gameState=STATE_GAME_GAME;
		}
		else if(gameState == STATE_GAME_OUTRO)
		{
			if(!cutScene.IsFinished() )
			{
				cutScene.Update();

				if(Keys::IsPressed(SDLK_ESCAPE) )
					cutScene.Skip();
				return;
			}
			else
			{
				NextLevel();
				return;
			}
		}*/
		if(!queueCutscenes.empty() || !cutScene.IsFinished() )
		{
			//printf("A");
			if(cutScene.IsFinished() )
			{
				//printf("B");
				/*queueCutscenes.pop_front();
				if(!queueCutscenes.empty() )
				{
					cutScene.Load(queueCutscenes.front() );
					printf("C");
				}*/
				cutScene.Load(queueCutscenes.front() );
				queueCutscenes.pop_front();
			}
			else
			{
				cutScene.Update();
				if(Keys::IsPressed(SDLK_ESCAPE) )
					cutScene.Skip();
				//printf("D");
				if(cutScene.IsScreenFilled() && needNextLevel)
				{
					//if(nextLevel != "end")
					if(!nextLevel.empty() )
					{
						NextLevel();
						needNextLevel=false;
					}
					//printf("E");
				}
			}
		}
		else if(needNextLevel)
		{
			//printf("F");
			NextLevel();
			needNextLevel=false;
		}
		else
		{
			/*
			 * END OF LEVEL~!?
			 */
			if(targetsCount == 0 && player)
			{
				/*float dx=nextLevelPos.x-player->GetCenter().x;
				float dy=nextLevelPos.y-player->GetCenter().y;
				float dist=sqrt(dx*dx+dy*dy);
				if(dist < 16)*/
				if(Level::CheckCollision(player->GetRect()+player->GetPosition(), nextLevelPos) )
				{
					if(!cutOutro.empty() )
					{
						//cutScene.Load(cutOutro);
						queueCutscenes.push_back(cutOutro);
						needNextLevel=true;
						//gameState=STATE_GAME_OUTRO;
					}
					else
						needNextLevel=true;
						//NextLevel();
				}
			}

			if( (alertTimerStarted && targetsCount > 0) || alertTimer > 10000)
				alertTimer+=delta*1000.0f;

			if(alertTimer > 20000)
			{
				while(!enemyAlert.empty() )
				{
					EnemyAlert &a=enemyAlert.front();

					Enemy *e=new Enemy;
					if(!e->Init(a.path, a.pos, GetDirection(a.dir), a.name, a.life, GetWeaponType(a.weapon), a.ammo) )
					{
						printf("Can't create enemy~!");
						delete e;
						enemyAlert.pop_front();
						continue;
					}

					float dist=enemyAlertDist;
					if(player)
					{
						Vector delta=e->GetCenter()-player->GetCenter();
						dist=sqrt(delta.x*delta.x+delta.y*delta.y);
					}
					if(dist >= enemyAlertDist)
						AddObject(e);
					else
					{
						e->Quit();
						delete e;
					}
					enemyAlert.pop_front();
				}
			}

			//printf("AlertTimer: %d\n", alertTimer);

			for(unsigned a=0; a < objs.size(); )
			{
				Object *o=objs[a];
				//if(o->GetLife() != -1.0f) // XXX TODO ???
				o->Update();
				if(o->GetLife() == 0.0f)
				{
					o->OnDead(); // < change life to -1.0f to leave "corpse"
					if(o->GetLife() == 0.0f)
					{
						o->Quit();
						delete o;
						objs.erase(objs.begin()+a);
					}
				}
				else
					a++;
			}

			static bool kPressed=false;
			if(Keys::IsPressed(SDLK_TAB) )
			{
				if(!kPressed)
				{
					kPressed=true;
					drawLifes=!drawLifes;
				}
			}
			else
				kPressed=false;
		}
	}

	void Show()
	{
		if(mainState != STATE_MAIN_GAME)
			return;

		Vector cam=Video::GetCamera();
		Size wndSize=Video::GetSize();
		int sx, sy, ex, ey;

		/*
		 * Draw background
		 */
		Video::DrawRect(Rect(0, 0, levelW*32, levelH*32), Z_BACKGROUND, Color(0.0f, 0.0f, 0.0f, 1.0f) );
		/*
		 * Draw grass
		 */
		for(int y=-wndSize.h/2; y < levelH*32+wndSize.h/2; y+=grass.GetHeight() )
		{
			for(int x=-wndSize.w/2; x < levelW*32+wndSize.w/2; x+=grass.GetWidth() )
			{
				grass.Show(Rect(0, 0, grass.GetWidth(), grass.GetHeight() ), Vector(x, y, Z_BACKGROUND) );
			}
		}

		sx=cam.x/32;
		sy=cam.y/32;
		ex=sx+(wndSize.w)/32+1;
		ey=sy+(wndSize.h)/32+1;
		if(sx < 0)
			sx=0;
		if(sy < 0)
			sy=0;
		if(ex >= (signed)levelW)
			ex=levelW-1;
		if(ey >= (signed)levelH)
			ey=levelH-1;

		Rect srcRect;
		srcRect.w=32; srcRect.h=32;
		for(unsigned y=sy; y <= (unsigned)ey; ++y)
		{
			for(unsigned x=sx; x <= (unsigned)ex; ++x)
			{
				unsigned type=level[y*levelW+x];

				srcRect.x=(type*32)%(int)tiles.GetWidth();
				srcRect.y=(type*32)/(int)tiles.GetWidth()*32;

				tiles.Show(srcRect, Vector(x*32, y*32, Z_LEVEL) );
			}
		}

		// XXX show end of level
		if(targetsCount == 0)
		{
			Video::DrawGUI(true);
			//Video::DrawRect(Rect(nextLevelPos.x-15-Video::GetCamera().x, nextLevelPos.y-15-Video::GetCamera().y, 32, 32), Color(1.0f, 0.0f, 1.0f, 1.0f) );
			//Video::DrawRect(Rect(nextLevelPos.x-Video::GetCamera().x, nextLevelPos.y-Video::GetCamera().y, nextLevelPos.w, nextLevelPos.h), Color(1.0f, 0.0f, 1.0f, 1.0f) );
			Video::DrawRect(Rect(nextLevelPos.x-Video::GetCamera().x, nextLevelPos.y-Video::GetCamera().y, nextLevelPos.w, nextLevelPos.h), Color(0.0f, 1.0f, 0.0f, fabs(sin(GetTicks()/500.0f) )/2.0f) );
			Video::DrawGUI(false);
		}

		for(auto itr : objs)
		  	itr->Show();

		//if(Keys::IsPressed(SDLK_TAB) )
		if(drawLifes)
			for(auto itr : objs)
				itr->ShowLife();

		/*
		 * Draw HUD
		 */
		HUD::Show();

		if(!cutScene.IsFinished() )
		{
			//cursorInSight=false;
			cursorColor=Color(1.0f, 1.0f, 1.0f, 1.0f);
			cutScene.Show();
		}

		/*Video::DrawGUI(true);
		int mx, my;
		Mouse::GetPosition(mx, my);
		cursor.Show(Rect(0, 0, cursor.GetWidth(), cursor.GetHeight() ), Vector(mx-cursor.GetWidth()/2, my-cursor.GetHeight()/2), (cursorInSight) ? Color(0.0f, 1.0f, 0.0f, 1.0f) : Color(1.0f, 0.0f, 0.0f, 1.0f) );
		Video::DrawGUI(false);*/
	}
}
