/*
 * Player.cpp
 *
 *  Created on: 24 sie 2013
 *     Project: LDJam27
 *      Author: cybek
 */

#include"Player.h"
#include"Input.h"
#include"Video.h"
#include"Level.h"
#include"Particle.h"

#include<cmath>
#include<cstdio>

SDLKey Player::cUp=SDLK_w, Player::cDown=SDLK_s, Player::cLeft=SDLK_a, Player::cRight=SDLK_d;//, Player::cFire=SDLK_SPACE;
unsigned Player::cFire=SDL_BUTTON_LEFT;
float stdPlayerLife=30.0f;

namespace
{
	unsigned savedAmmo[WEAPON_COUNT]={0,0,0,0,0};
	unsigned lastCurWeapon=WEAPON_PISTOL;
}

Player::Player() : Object("Player"), move(false), speed(0.0f), curWeapon(0)
{

}

Player::~Player()
{

}

bool Player::Init(Vector nPos, float nSpeed, bool *nWeaponsEnabled)
{
	if(!spr.Load("mc.spr") )
		return false;

	if(!weapons[0].Init(this, WEAPON_PISTOL, Vector(13, 6) ) )
		return false;
	if(!weapons[1].Init(this, WEAPON_PPSZ, Vector(13, 6) ) )
		return false;
	if(!weapons[2].Init(this, WEAPON_RCLR, Vector(13, 6) ) )
		return false;
	if(!weapons[3].Init(this, WEAPON_CROSSBOW, Vector(13, 6) ) )
		return false;
	if(!weapons[4].Init(this, WEAPON_EGGPLANT, Vector(13, 6) ) )
		return false;

	for(unsigned a=0; a < WEAPON_COUNT; ++a)
	{
		if(weapons[a].GetAmmo() < savedAmmo[a] )
			weapons[a].SetAmmo(savedAmmo[a] );
	}

	event.Register(SDL_MOUSEBUTTONDOWNMASK);
	event.Register(SDL_KEYDOWNMASK);

	pos=nPos;
	pos.z=Z_PLAYER;
	curWeapon=0;
	//speed=3.0f;
	speed=nSpeed;

	//for(auto &itr : weaponsEnabled)
	//	itr=false;
	bool set=false;
	for(unsigned a=0; a < WEAPON_COUNT; ++a)
	{
		weaponsEnabled[a]=nWeaponsEnabled[a];
		if(!set && weaponsEnabled[a] )
		{
			curWeapon=a;
			set=true;
		}
	}

	if(weaponsEnabled[lastCurWeapon] )
		curWeapon=lastCurWeapon;

	maskType=COL_RADIAL;
	mask.x=4; mask.y=4;
	mask.w=24; mask.h=24;

	SetLife(stdPlayerLife);

	return true;
}

void Player::Quit()
{
	lastCurWeapon=curWeapon;
	for(unsigned a=0; a < WEAPON_COUNT; ++a)
		savedAmmo[a]=weapons[a].GetAmmo();

	for(auto &itr : weapons)
		itr.Quit();

	spr.Free();
}

void Player::Update()
{
	Vector vel;

	move=true;
	if(Keys::IsPressed(cUp) )
	{
		if(Keys::IsPressed(cLeft) )
			dir=DIR_UP_LEFT;
		else if(Keys::IsPressed(cRight) )
			dir=DIR_UP_RIGHT;
		else
			dir=DIR_UP;
	}
	else if(Keys::IsPressed(cDown) )
	{
		if(Keys::IsPressed(cLeft) )
			dir=DIR_DOWN_LEFT;
		else if(Keys::IsPressed(cRight) )
			dir=DIR_DOWN_RIGHT;
		else
			dir=DIR_DOWN;
	}
	else if(Keys::IsPressed(cLeft) )
		dir=DIR_LEFT;
	else if(Keys::IsPressed(cRight) )
		dir=DIR_RIGHT;
	else
	{
		move=false;
		vel.x=0.0f;
		vel.y=0.0f;
	}

	if(!move) // XXX Find best dir
	{
		int mx, my;
		Mouse::GetPosition(mx, my);
		Vector cam=Video::GetCamera();

		float rad=atan2( (my+cam.y)-(pos.y+mask.y+mask.h/2), (mx+cam.x)-(pos.x+mask.x+mask.w/2) );
		float angle=rad/3.14f*180.0f;
		//if(angle < 0)
		//	angle+=360;
		//printf("Angle: %.1f\n", angle);
		/*if(angle < 23 && angle > 338)
			dir=DIR_RIGHT;
		else if(angle < 68 && angle >= 23)
			dir=DIR_DOWN_RIGHT;
		else if(angle < 113 && angle >= 68)
			dir=DIR_DOWN;
		else if(angle < 158 && angle >= 113)
			dir=DIR_DOWN_LEFT;
		else if(angle < 203 && angle >= 158)
			dir=DIR_LEFT;
		else if(angle < 248 && angle >= 203)
			dir=DIR_UP_LEFT;
		else if(angle < 293 && angle >= 248)
			dir=DIR_UP;
		else if(angle < 338 && angle >= 293)
			dir=DIR_UP_RIGHT;*/

		if(angle < 23 && angle > -23)
			dir=DIR_RIGHT;
		else if(angle < 68 && angle >= 23)
			dir=DIR_DOWN_RIGHT;
		else if(angle < 113 && angle >= 68)
			dir=DIR_DOWN;
		else if(angle < 158 && angle >= 113)
			dir=DIR_DOWN_LEFT;
		else if(angle < -158 || angle >= 158)
			dir=DIR_LEFT;
		else if(angle < -113 && angle >= -158)
			dir=DIR_UP_LEFT;
		else if(angle < -68 && angle >= -113)
			dir=DIR_UP;
		else if(angle < -23 && angle >= -68)
			dir=DIR_UP_RIGHT;
	}

	switch(dir)
	{
		case DIR_UP:
		{
			spr.SetAnimation("up");

			if(move)
			{
				vel.x=0.0f;
				vel.y=-1.0f;
			}

			break;
		}
		case DIR_DOWN:
		{
			spr.SetAnimation("down");

			if(move)
			{
				vel.x=0.0f;
				vel.y=1.0f;
			}

			break;
		}
		case DIR_LEFT:
		{
			spr.SetAnimation("left");

			if(move)
			{
				vel.x=-1.0f;
				vel.y=0.0f;
			}

			break;
		}
		case DIR_RIGHT:
		{
			spr.SetAnimation("right");

			if(move)
			{
				vel.x=1.0f;
				vel.y=0.0f;
			}

			break;
		}
		case DIR_UP_LEFT:
		{
			spr.SetAnimation("up_left");

			if(move)
			{
				vel.x=-1.0f;
				vel.y=-1.0f;
			}

			break;
		}
		case DIR_UP_RIGHT:
		{
			spr.SetAnimation("up_right");

			if(move)
			{
				vel.x=1.0f;
				vel.y=-1.0f;
			}

			break;
		}
		case DIR_DOWN_LEFT:
		{
			spr.SetAnimation("down_left");

			if(move)
			{
				vel.x=-1.0f;
				vel.y=1.0f;
			}

			break;
		}
		case DIR_DOWN_RIGHT:
		{
			spr.SetAnimation("down_right");

			if(move)
			{
				vel.x=1.0f;
				vel.y=1.0f;
			}

			break;
		}
	}

	/*static bool f=true;
	if(Keys::IsPressed(cFire) || Mouse::IsPressed(SDL_BUTTON_LEFT) )
	{
		if(f)
			weapons[curWeapon].Fire();
		f=false;
	}
	else
		f=true;*/

	/*if(Keys::IsPressed(cFire) || Mouse::IsPressed(SDL_BUTTON_LEFT) )
	{
		//if(!weapons[curWeapon].Fire() );
		weapons[curWeapon].Fire();
			//printf("Can't fire weapon!\n");
	}*/

	for(auto e : event)
	{
		//printf("PLAYER EVENT~!\n");
		if(e.type == SDL_MOUSEBUTTONDOWN)
		{
			if(e.button.button == cFire)
			{
				//if(curWeapon != WEAPON_PPSZ)
				if(weapons[curWeapon].GetFireDelay() == 0)
					weapons[curWeapon].Fire();
			}
			else if(e.button.button == SDL_BUTTON_WHEELDOWN)
			{
				do
				{
					if(curWeapon == 0)
						curWeapon=WEAPON_COUNT-1;
					else
						curWeapon--;
				} while(!weaponsEnabled[curWeapon] );
			}
			else if(e.button.button == SDL_BUTTON_WHEELUP)
			{
				do
				{
					curWeapon++;
					if(curWeapon == WEAPON_COUNT)
						curWeapon=0;
				} while(!weaponsEnabled[curWeapon] );
			}
		}
		else if(e.type == SDL_KEYDOWN)
		{
			if(e.key.keysym.sym == SDLK_1 && weaponsEnabled[WEAPON_PISTOL] )
				curWeapon=WEAPON_PISTOL;
			else if(e.key.keysym.sym == SDLK_2 && weaponsEnabled[WEAPON_PPSZ] )
				curWeapon=WEAPON_PPSZ;
			else if(e.key.keysym.sym == SDLK_3 && weaponsEnabled[WEAPON_RCLR] )
				curWeapon=WEAPON_RCLR;
			else if(e.key.keysym.sym == SDLK_4 && weaponsEnabled[WEAPON_CROSSBOW] )
				curWeapon=WEAPON_CROSSBOW;
			else if(e.key.keysym.sym == SDLK_5 && weaponsEnabled[WEAPON_EGGPLANT] )
				curWeapon=WEAPON_EGGPLANT;
		}
	}

	//if(curWeapon == WEAPON_PPSZ && Mouse::IsPressed(cFire) )
	if(weapons[curWeapon].GetFireDelay() > 0 && Mouse::IsPressed(cFire) )
		weapons[curWeapon].Fire();

	vel.x*=speed;
	vel.y*=speed;

	Level::MoveObject(this, vel, false, NULL);

	Size wnd=Video::GetSize();
	Video::SetCamera(pos+Vector(mask.x+mask.w/2-wnd.w/2, mask.y+mask.h/2-wnd.h/2) );

	weapons[curWeapon].Update();

	/*int mx, my;
	Mouse::GetPosition(mx, my);
	float rad=atan2( (my+Video::GetCamera().y)-(pos.y+rect.y+rect.h/2), (mx+Video::GetCamera().x)-(pos.x+rect.x+rect.w/2) );
	rad+=3.14/2.0f;
	if(rad > 3.14f)
		rad-=3.14f*2.0f;
	if(rad < -3.14f)
		rad+=3.14f*2.0f;
	printf("Rad: %.2f, Angle: %.1f\n", rad, rad*180.0f/3.14f);*/

	//printf("%.02f %.02f\t\t\r", pos.x, pos.y);
}

void Player::Show()
{
	weapons[curWeapon].Show();
	spr.Show(pos);
}

void Player::OnDead()
{
	// TODO game over~!
	Level::player=NULL;

	unsigned bloodAmount=600;
	for(unsigned a=0; a < bloodAmount; ++a)
	{
		Particle *p=new Particle;
		if(!p->Init(pos+Vector(mask.x+mask.w/2.0f, mask.y+mask.h/2.0f), (rand()%40)/25.0f+1.0f, rand()%360, (mask.w/2.0f+mask.h/2.0f)+5, Color(1.0f, 0.0f, 0.0f, 1.0f), Color(0.6f, 0.0f, 0.0f, 0.0f) ) )
			printf("Can't create particle!\n");
		Level::AddObject(p);
	}
}

unsigned Player::GetCurrentWeapon()
{
	return curWeapon;
}

bool Player::HasWeapon(WEAPON_TYPE type)
{
	return weaponsEnabled[type];
}

unsigned Player::GetAmmo(WEAPON_TYPE type)
{
	return weapons[type].GetAmmo();
}

unsigned Player::GetAmmoMax(WEAPON_TYPE type)
{
	return weapons[type].GetAmmoMax();
}

void Player::AddAmmo(WEAPON_TYPE type, unsigned amount)
{
	if(type == WEAPON_NONE)
		return;

	weapons[type].AddAmmo(amount);
}
