/*
 * Sprite.cpp
 *
 *  Created on: 23 sie 2013
 *     Project: LDJam27
 *      Author: cybek
 */

#include"Sprite.h"
#include<fstream>
#include<vector>
#include<map>

struct SpriteData
{
	struct Frame
	{
		Rect src;
	};
	struct Animation
	{
		std::vector<Frame> frames;
		unsigned delay;
		bool loop;
	};

	std::vector<Animation> anims;
	std::map<String, unsigned> animsNames;
	Texture tex;
	unsigned usage;

	SpriteData() : usage(1) {}
};

namespace
{
	std::map<String, SpriteData*> loadedSprs;
}

Sprite::Sprite() : data(NULL), curFrame(0), curAnim(0), timer(0), angle(0.0f)
{

}

Sprite::~Sprite()
{
	if(data)
		printf("Sprite not freed: %s (%d)\n", path.c_str(), data->usage);
}

bool Sprite::Load(String path)
{
	auto itr=loadedSprs.find(path);
	if(itr != loadedSprs.end() )
	{
		data=itr->second;
		data->usage++;
		this->path=path;
		return true;
	}

	//printf("Loading sprite: %s\n", path.c_str() );

	std::ifstream f( ("data/"+path).c_str() );
	if(!f)
		return false;
	data=new SpriteData;

	std::string texPath;
	f >> texPath;

	data->tex.Load(texPath);

	std::string cmd;
	while(true)
	{
		f >> cmd;
		if(f.fail() )
			break;

		if(cmd == "anim")
		{
			std::string name;
			unsigned delay, loop;

			f >> name >> delay >> loop;

			data->animsNames[name]=data->anims.size();
			data->anims.push_back(SpriteData::Animation() );
			data->anims.back().delay=delay;
			data->anims.back().loop=loop;
		}
		else if(cmd == "frame")
		{
			float x, y, w, h;
			f >> x >> y >> w >> h;

			data->anims.back().frames.push_back(SpriteData::Frame() );
			data->anims.back().frames.back().src.x=x;
			data->anims.back().frames.back().src.y=y;
			data->anims.back().frames.back().src.w=w;
			data->anims.back().frames.back().src.h=h;
		}
		else
		{
			printf("Unknown spr param: %s\n", cmd.c_str() );
			return false;
		}

		if(f.fail() )
			return false;
	}

	loadedSprs.insert(std::pair<String, SpriteData*>(path, data) );

	this->path=path;
	return true;
}

void Sprite::Free()
{
	for(auto itr=loadedSprs.begin(); itr != loadedSprs.end(); ++itr)
	{
		if(data == itr->second)
		{
			data->usage--;
			if(data->usage == 0)
			{
				data->tex.Free();
				delete data;
				loadedSprs.erase(itr);
			}

			break;
		}
	}

	data=NULL;
}

void Sprite::Update()
{
	SpriteData::Animation &anim=data->anims[curAnim];

	timer+=delta*1000.0f;
	if(timer >= anim.delay)
	{
		timer=0;
		curFrame++;
		if(curFrame >= anim.frames.size() )
			curFrame=0;
	}
}

void Sprite::Show(Vector dst)
{
	Rect &src=data->anims[curAnim].frames[curFrame].src;

	if(angle == 0.0f && angleSpot.x == 0.0f && angleSpot.y == 0.0f)
		data->tex.Show(src, dst, color);
	else
		data->tex.Show(src, dst, angle, angleSpot, color);
}

void Sprite::SetAnimation(String name)
{
	auto itr=data->animsNames.find(name);
	if(itr == data->animsNames.end() )
	{
		printf("Can't find animation: %s\n", name.c_str() );
		return;
	}

	//curAnim=data->animsNames[name];
	curAnim=itr->second;
	curFrame=0;
	timer=0;
}

String Sprite::GetAnimation()
{
	for(auto itr : data->animsNames)
	{
		if(itr.second == curAnim)
			return itr.first;
	}

	return "";
}

unsigned Sprite::GetAnimationsCount()
{
	return data->anims.size();
}

void Sprite::SetFrame(unsigned id)
{
	curFrame=id;
	timer=0;
}

unsigned Sprite::GetFrame()
{
	return curFrame;
}

void Sprite::SetAngle(float nAngle, Vector nAngleSpot)
{
	angle=nAngle;
	angleSpot=nAngleSpot;

	if(angle < 0)
		angle+=360;
	if(angle >= 360)
		angle-=360;
}

float Sprite::GetAngle()
{
	return angle;
}

Vector Sprite::GetAngleSpot()
{
	return angleSpot;
}

void Sprite::SetColor(Color nColor)
{
	color=nColor;
}

Rect Sprite::GetMask() // OBSTACLE ONLY~!
{
	return data->anims[0].frames[0].src;
}
