/*
 * Texture.cpp
 *
 *  Created on: 23 sie 2013
 *     Project: LDJam27
 *      Author: cybek
 */

#include"Texture.h"
//#include<SOIL.h>
#include"SOIL/SOIL.h"
#include<SDL/SDL_opengl.h>
#include<cstdio>
#include<cmath>
#include<map>

namespace
{
	unsigned boundID=0;

	class Info
	{
		public:
			unsigned id, usage;
			int w, h;

			Info(unsigned id, int w, int h) : id(id), usage(1), w(w), h(h) {}
			Info(const Info &i) : id(i.id), usage(i.usage), w(i.w), h(i.h) {}
	};
	std::map<String, Info> loadedTexs;
}

Texture::Texture() : id(0)
{

}

Texture::~Texture()
{
	if(id > 0)
		printf("Texture not freed: %s (%d)\n", path.c_str(), id);
}

bool Texture::Load(String path)
{
	auto itr=loadedTexs.find(path);
	if(itr != loadedTexs.end() )
	{
		id=itr->second.id;
		size.w=itr->second.w;
		size.h=itr->second.h;
		itr->second.usage++;
		this->path=path;
		return true;
	}

	//printf("Loading texture: %s\n", path.c_str() );

	int w, h;
	id=SOIL_load_OGL_texture( ("data/"+path).c_str(), SOIL_LOAD_RGBA, SOIL_CREATE_NEW_ID, 0, &w, &h);
	if(!id)
		return 0;
	size.w=w;
	size.h=h;
	this->path=path;

	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	//loadedTexs[path]=Info(id, w, h);
	loadedTexs.insert(std::pair<String, Info>(path, Info(id, w, h) ) );

	return true;
}

void Texture::Free()
{
	for(auto itr=loadedTexs.begin(); itr != loadedTexs.end(); ++itr)
	{
		if(id == itr->second.id)
		{
			itr->second.usage--;
			if(itr->second.usage == 0)
			{
				glDeleteTextures(1, &id);
				loadedTexs.erase(itr);
			}

			break;
		}
	}

	id=0;
}

void Texture::Show(Rect src, Vector dst)
{
	if(boundID != id)
	{
		glBindTexture(GL_TEXTURE_2D, id);
		boundID=id;
	}

	float tx1, ty1, tx2, ty2;
	tx1=src.x/size.w;
	ty1=src.y/size.h;
	tx2=(src.x+src.w)/size.w;
	ty2=(src.y+src.h)/size.h;

	glBegin(GL_QUADS);

	glTexCoord2f(tx1, ty1);
	glVertex3i(dst.x, dst.y, dst.z);

	glTexCoord2f(tx2, ty1);
	glVertex3i(dst.x+src.w, dst.y, dst.z);

	glTexCoord2f(tx2, ty2);
	glVertex3i(dst.x+src.w, dst.y+src.h, dst.z);

	glTexCoord2f(tx1, ty2);
	glVertex3i(dst.x, dst.y+src.h, dst.z);

	glEnd();
}

void Texture::Show(Rect src, Vector dst, Color c)
{
	if(boundID != id)
	{
		glBindTexture(GL_TEXTURE_2D, id);
		boundID=id;
	}

	float tx1, ty1, tx2, ty2;
	tx1=src.x/size.w;
	ty1=src.y/size.h;
	tx2=(src.x+src.w)/size.w;
	ty2=(src.y+src.h)/size.h;

	glColor4f(c.r, c.g, c.b, c.a);
	glBegin(GL_QUADS);

	glTexCoord2f(tx1, ty1);
	glVertex3i(dst.x, dst.y, dst.z);

	glTexCoord2f(tx2, ty1);
	glVertex3i(dst.x+src.w, dst.y, dst.z);

	glTexCoord2f(tx2, ty2);
	glVertex3i(dst.x+src.w, dst.y+src.h, dst.z);

	glTexCoord2f(tx1, ty2);
	glVertex3i(dst.x, dst.y+src.h, dst.z);

	glEnd();
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

void Texture::Show(Rect src, Vector dst, float angle, Vector angleSpot)
{
	if(boundID != id)
	{
		glBindTexture(GL_TEXTURE_2D, id);
		boundID=id;
	}

	float tx1, ty1, tx2, ty2;
	tx1=src.x/size.w;
	ty1=src.y/size.h;
	tx2=(src.x+src.w)/size.w;
	ty2=(src.y+src.h)/size.h;

	Vector p1, p2, p3, p4;

	p1.x=-src.w/2.0f+angleSpot.x;
	p1.y=-src.h/2.0f+angleSpot.y;
	p2.x=src.w/2.0f+angleSpot.x;
	p2.y=-src.h/2.0f+angleSpot.y;
	p3.x=src.w/2.0f+angleSpot.x;
	p3.y=src.h/2.0f+angleSpot.y;
	p4.x=-src.w/2.0f+angleSpot.x;
	p4.y=src.h/2.0f+angleSpot.y;

	float rad=angle*3.14f/180.0f;
	Vector pn1, pn2, pn3, pn4;
	/*pn1.x=round( p1.x*cos(rad)-p1.y*sin(rad)+src.w/2 )-angleSpot.x;
	pn1.y=round( p1.x*sin(rad)+p1.y*cos(rad)+src.h/2 )-angleSpot.y;
	pn2.x=round( p2.x*cos(rad)-p2.y*sin(rad)+src.w/2 )-angleSpot.x;
	pn2.y=round( p2.x*sin(rad)+p2.y*cos(rad)+src.h/2 )-angleSpot.y;
	pn3.x=round( p3.x*cos(rad)-p3.y*sin(rad)+src.w/2 )-angleSpot.x;
	pn3.y=round( p3.x*sin(rad)+p3.y*cos(rad)+src.h/2 )-angleSpot.y;
	pn4.x=round( p4.x*cos(rad)-p4.y*sin(rad)+src.w/2 )-angleSpot.x;
	pn4.y=round( p4.x*sin(rad)+p4.y*cos(rad)+src.h/2 )-angleSpot.y;*/
	pn1.x=round( p1.x*cos(rad)-p1.y*sin(rad)+src.w/2 );
	pn1.y=round( p1.x*sin(rad)+p1.y*cos(rad)+src.h/2 );
	pn2.x=round( p2.x*cos(rad)-p2.y*sin(rad)+src.w/2 );
	pn2.y=round( p2.x*sin(rad)+p2.y*cos(rad)+src.h/2 );
	pn3.x=round( p3.x*cos(rad)-p3.y*sin(rad)+src.w/2 );
	pn3.y=round( p3.x*sin(rad)+p3.y*cos(rad)+src.h/2 );
	pn4.x=round( p4.x*cos(rad)-p4.y*sin(rad)+src.w/2 );
	pn4.y=round( p4.x*sin(rad)+p4.y*cos(rad)+src.h/2 );

	/*printf("Angle: %.1f\n", angle);
	printf("  P1: %.1fx%.1f\n", pn1.x, pn1.y);
	printf("  P2: %.1fx%.1f\n", pn2.x, pn2.y);
	printf("  P3: %.1fx%.1f\n", pn3.x, pn3.y);
	printf("  P4: %.1fx%.1f\n", pn4.x, pn4.y);*/

	glBegin(GL_QUADS);

	glTexCoord2f(tx1, ty1);
	glVertex3i(pn1.x+dst.x, pn1.y+dst.y, dst.z);

	glTexCoord2f(tx2, ty1);
	//glVertex2i(dst.x+src.w, dst.y);
	glVertex3i(pn2.x+dst.x, pn2.y+dst.y, dst.z);

	glTexCoord2f(tx2, ty2);
	//glVertex2i(dst.x+src.w, dst.y+src.h);
	glVertex3i(pn3.x+dst.x, pn3.y+dst.y, dst.z);

	glTexCoord2f(tx1, ty2);
	//glVertex2i(dst.x, dst.y+src.h);
	glVertex3i(pn4.x+dst.x, pn4.y+dst.y, dst.z);

	glEnd();
}

void Texture::Show(Rect src, Vector dst, float angle, Vector angleSpot, Color c)
{
	//return;

	if(boundID != id)
	{
		glBindTexture(GL_TEXTURE_2D, id);
		boundID=id;
	}

	float tx1, ty1, tx2, ty2;
	tx1=src.x/size.w;
	ty1=src.y/size.h;
	tx2=(src.x+src.w)/size.w;
	ty2=(src.y+src.h)/size.h;

	Vector p1, p2, p3, p4;

	p1.x=-src.w/2.0f+angleSpot.x;
	p1.y=-src.h/2.0f+angleSpot.y;
	p2.x=src.w/2.0f+angleSpot.x;
	p2.y=-src.h/2.0f+angleSpot.y;
	p3.x=src.w/2.0f+angleSpot.x;
	p3.y=src.h/2.0f+angleSpot.y;
	p4.x=-src.w/2.0f+angleSpot.x;
	p4.y=src.h/2.0f+angleSpot.y;

	float rad=angle*3.14f/180.0f;
	Vector pn1, pn2, pn3, pn4;
	pn1.x=round( p1.x*cos(rad)-p1.y*sin(rad)+src.w/2 );
	pn1.y=round( p1.x*sin(rad)+p1.y*cos(rad)+src.h/2 );
	pn2.x=round( p2.x*cos(rad)-p2.y*sin(rad)+src.w/2 );
	pn2.y=round( p2.x*sin(rad)+p2.y*cos(rad)+src.h/2 );
	pn3.x=round( p3.x*cos(rad)-p3.y*sin(rad)+src.w/2 );
	pn3.y=round( p3.x*sin(rad)+p3.y*cos(rad)+src.h/2 );
	pn4.x=round( p4.x*cos(rad)-p4.y*sin(rad)+src.w/2 );
	pn4.y=round( p4.x*sin(rad)+p4.y*cos(rad)+src.h/2 );
	/*pn1.x=round( p1.x*cos(rad)-p1.y*sin(rad)+src.w/2 )-angleSpot.x;
	pn1.y=round( p1.x*sin(rad)+p1.y*cos(rad)+src.h/2 )-angleSpot.y;
	pn2.x=round( p2.x*cos(rad)-p2.y*sin(rad)+src.w/2 )-angleSpot.x;
	pn2.y=round( p2.x*sin(rad)+p2.y*cos(rad)+src.h/2 )-angleSpot.y;
	pn3.x=round( p3.x*cos(rad)-p3.y*sin(rad)+src.w/2 )-angleSpot.x;
	pn3.y=round( p3.x*sin(rad)+p3.y*cos(rad)+src.h/2 )-angleSpot.y;
	pn4.x=round( p4.x*cos(rad)-p4.y*sin(rad)+src.w/2 )-angleSpot.x;
	pn4.y=round( p4.x*sin(rad)+p4.y*cos(rad)+src.h/2 )-angleSpot.y;*/

	/*printf("Angle: %.1f\n", angle);
	printf("  P1: %.1fx%.1f\n", pn1.x, pn1.y);
	printf("  P2: %.1fx%.1f\n", pn2.x, pn2.y);
	printf("  P3: %.1fx%.1f\n", pn3.x, pn3.y);
	printf("  P4: %.1fx%.1f\n", pn4.x, pn4.y);*/

	glColor4f(c.r, c.g, c.b, c.a);
	glBegin(GL_QUADS);

	glTexCoord2f(tx1, ty1);
	glVertex3i(pn1.x+dst.x, pn1.y+dst.y, dst.z);

	glTexCoord2f(tx2, ty1);
	//glVertex2i(dst.x+src.w, dst.y);
	glVertex3i(pn2.x+dst.x, pn2.y+dst.y, dst.z);

	glTexCoord2f(tx2, ty2);
	//glVertex2i(dst.x+src.w, dst.y+src.h);
	glVertex3i(pn3.x+dst.x, pn3.y+dst.y, dst.z);

	glTexCoord2f(tx1, ty2);
	//glVertex2i(dst.x, dst.y+src.h);
	glVertex3i(pn4.x+dst.x, pn4.y+dst.y, dst.z);

	glEnd();
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
}

Size Texture::GetSize()
{
	return size;
}

float Texture::GetWidth()
{
	return size.w;
}

float Texture::GetHeight()
{
	return size.h;
}
