/*
 * Video.cpp
 *
 *  Created on: 23 sie 2013
 *     Project: LDJam27
 *      Author: cybek
 */

#include"Video.h"
#include<SDL/SDL.h>
#include<SDL/SDL_opengl.h>

float delta=0.0f;
unsigned deltaTimer=0;

namespace Video
{
	Size wndSize;
	Vector camera;

	//unsigned frame=0, frameTimer=0;

	bool Init(Size size, String caption)
	{
		SDL_Init(SDL_INIT_VIDEO);
		SDL_WM_SetCaption(caption.c_str(), NULL);
		SDL_ShowCursor(false);
		SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, 1);
		if(!SDL_SetVideoMode(size.w, size.h, 32, SDL_OPENGL) )
			return false;

		glClearColor(1.0f, 1.0f, 1.0f, 1.0f);

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0, size.w, size.h, 0, -100, 100);

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();

		glEnable(GL_TEXTURE_2D);

		//glDisable(GL_DEPTH_TEST);
		glEnable(GL_DEPTH_TEST);

		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

		glEnable(GL_ALPHA_TEST);
		glAlphaFunc(GL_GREATER, 0.0f);

		GLenum err=glGetError();
		if(err != GL_NO_ERROR)
		{
			printf("OpenGL error: %x\n", err);
			return false;
		}
		wndSize=size;
		return true;
	}

	void Quit()
	{
		SDL_Quit();
	}

	void Flip()
	{
		SDL_GL_SwapBuffers();
		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

		/*if(SDL_GetTicks()-deltaTimer  < 1000/60)
		{
			printf("Too fast\n");
			SDL_Delay(17);
			delta=1.0f/60.0f;
		}
		else
		{
			printf("OK\n");
			delta=(SDL_GetTicks()-deltaTimer)/1000.0f;
			deltaTimer=SDL_GetTicks();
		}*/

		delta=(SDL_GetTicks()-deltaTimer)/1000.0f;
		deltaTimer=SDL_GetTicks();
		if(delta > 0.1f)
			delta=0.1f;

		/*if(delta < 1.0f/70.0f)
		{
			//printf("Too fast~!\n");
			//printf("Delta: %.6f, 60delta: %.6f\n", delta, 1.0f/60.0f);
			//SDL_Delay(1000/60);
			SDL_Delay(16);
			delta=1.0f/60.0f;
		}*/
		/*delta=(SDL_GetTicks()-deltaTimer)/1000.0f;
		if(delta == 0.0f)
			SDL_Delay(1);
		else if(delta > 0.1f)
		{
			delta=0.1f;
			deltaTimer=SDL_GetTicks();
		}
		else
			deltaTimer=SDL_GetTicks();*/

		/*frame++;
		if(SDL_GetTicks() >= frameTimer+1000)
		{
			printf("FPS: %.1f\n", frame/( (SDL_GetTicks()-frameTimer)/1000.0f) );
			frame=0;
			frameTimer=SDL_GetTicks();
		}*/
	}

	Size GetSize()
	{
		return wndSize;
	}

	void SetCamera(Vector nCam)
	{
		glTranslatef(camera.x, camera.y, 0.0f);
		glTranslatef(-nCam.x, -nCam.y, 0.0f);

		camera=nCam;
	}

	Vector GetCamera()
	{
		return camera;
	}

	void DrawRect(Rect dst, Color c)
	{
		glDisable(GL_DEPTH_TEST);
		glDisable(GL_TEXTURE_2D);

		glColor4f(c.r, c.g, c.b, c.a);
		glBegin(GL_QUADS);

		glVertex2i(dst.x, dst.y);
		glVertex2i(dst.x+dst.w, dst.y);
		glVertex2i(dst.x+dst.w, dst.y+dst.h);
		glVertex2i(dst.x, dst.y+dst.h);

		glEnd();
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

		glEnable(GL_TEXTURE_2D);
		glEnable(GL_DEPTH_TEST);
	}

	void DrawRect(Rect dst, float z, Color c)
	{
		//glDisable(GL_DEPTH_TEST);
		glDisable(GL_TEXTURE_2D);

		glColor4f(c.r, c.g, c.b, c.a);
		glBegin(GL_QUADS);

		glVertex3i(dst.x, dst.y, z);
		glVertex3i(dst.x+dst.w, dst.y, z);
		glVertex3i(dst.x+dst.w, dst.y+dst.h, z);
		glVertex3i(dst.x, dst.y+dst.h, z);

		glEnd();
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

		glEnable(GL_TEXTURE_2D);
		//glEnable(GL_DEPTH_TEST);
	}

	void DrawLine(Vector v1, Vector v2, float width, Color c)
	{
		if(v1.z != v2.z)
		{
			//printf("NO LINE~!\n");
			return;
		}

		//glDisable(GL_DEPTH_TEST);
		glDisable(GL_TEXTURE_2D);

		glLineWidth(width);
		glColor4f(c.r, c.g, c.b, c.a);
		glBegin(GL_LINES);

		glVertex3i(v1.x, v1.y, v1.z);
		glVertex3i(v2.x, v2.y, v2.z);

		glEnd();
		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

		glEnable(GL_TEXTURE_2D);
		//glEnable(GL_DEPTH_TEST);
	}

	void DrawGUI(bool state)
	{
		static bool curState=false;
		if(curState == state)
			return;

		if(state)
		{
			glDisable(GL_DEPTH_TEST);
			glLoadIdentity();
			//glTranslatef(camera.x, camera.y, 0.0f);
		}
		else
		{
			glEnable(GL_DEPTH_TEST);
			glLoadIdentity();
			glTranslatef(-camera.x, -camera.y, 0.0f);
		}

		curState=state;
	}

	void SetCursor(Vector nPos)
	{
		SDL_WarpMouse(nPos.x, nPos.y);
	}

}
